import 'dart:async';
import 'dart:convert';
import 'dart:io';
import '../consts.dart';
import 'dart:typed_data';


/// Fetch relation data from server
Future<Uint8List> fetchRelation() async {
  try {
    // Custom HTTP client
    HttpClient client = HttpClient()
      ..badCertificateCallback = // NB: temporary disable SSL certificate validation
          (X509Certificate cert, String host, int port) => true;

    // Execute request to to get_relation endpoint
    var request = await client.getUrl(Uri.parse('${serverURI}get_relation'));
    var response = await request.close(); // Close response body at end of function

    // Parse body to JSON if request is ok
    if (response.statusCode == 200) {
      var responseBody = await response.transform(utf8.decoder).join();

      if (responseBody.isNotEmpty) {
        return Uint8List.fromList(utf8.encode(responseBody));
      } else {
        throw Exception('Response body is empty');
      }
    } else {
      throw Exception('Failed to fetch relation data: Status code ${response.statusCode}');
    }
  } catch (e) {
    throw Exception('Failed to fetch relation data: ${e.toString()}');
  }
}

