import string
import json
import geopandas as gpd
from server.map.get_markers import get_all_markers
from shapely.geometry import Polygon, LineString, Point


# get_relation returns the geojson data for a selected body of water
def get_relation(self, body_of_water: string):
    # Load GeoJSON data using geopandas
    geo_data = gpd.read_file("server/map/mjosa.geojson")

    # Filter only polygons, exclude points
    polygon_data = geo_data[geo_data['geometry'].geom_type == 'Polygon']

    # Convert GeoDataFrame to dictionary
    geojson_dict = json.loads(polygon_data.to_json())

    # Convert response data to JSON string
    response_json = json.dumps(divide_relation(self, polygon_data))

    # Set headers
    self.send_response(200)
    self.send_header("Content-type", "application/json")
    self.end_headers()

    # Write coordinates to response object
    self.wfile.write(response_json.encode('utf-8'))


def divide_relation(self, polygon):
    # Fetch measurements from database
    measurements = get_all_markers(self.cursor, False, 'Mjosa')

    # Temporary hardcoded point at which relation is split, should be
    splitting_points = [(60.6853, 10.9571), (60.7018, 11.1058)]

    # Lists to hold new polygons
    divided_relation = []
    temp_seg_1 = []
    temp_seg_2 = []

    # Loop through polygon coordinates and split at defined coordinates
    for i in range(len(polygon)):

        closest_point = []

        # Find the closest point on the polygon to the splitting point
        for point in splitting_points:
            closest_point = polygon.exterior.interpolate(polygon.exterior.project(Point(point)),normalized=True)

        # Check if the closest point is already in the polygon coordinates
        if closest_point.coords[0] not in polygon:
            # If not, add it to the polygon coordinates
            polygon.append(closest_point.coords[0])
        # Add the splitting point (either the original or the closest) to the splitting points list
        splitting_points.append(closest_point.coords[0])

        current_point = polygon[i]
        next_point = polygon[(i + 1) % len(polygon)]  # Circular index

        # Check if the current segment crosses any splitting point
        for splitting_point in splitting_points:
            if (current_point != splitting_point and
                    next_point != splitting_point and
                    LineString([current_point, next_point]).intersects(Point(splitting_point))):
                # Split the segment at the splitting point
                temp_seg_1.append(current_point)
                temp_seg_2.append(splitting_point)
                temp_seg_1.append(splitting_point)
                temp_seg_2.append(next_point)
                break
        else:
            # If the segment doesn't cross any splitting point, add it to both lists
            temp_seg_1.append(current_point)
            temp_seg_2.append(current_point)

    # Return a list containing all the created polygons
    divided_relation = [Polygon(temp_seg_1), Polygon(temp_seg_2)]

    return divided_relation
